//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//
//  " S T A T I S T I C S " plugin
//
//	Collects and lists some statistics about the current score.
//	Version 0.1 - Date 11.02.2010
//
//	By Maurizio M. Gavioli, 2010.
//	Derived from a plugin by lasconic.
//
//  MuseScore: Copyright (C)2008 Werner Schweer and others
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

// Arrays containing the number of occurrences and the total duration of each pitch
var g_numOfPitches = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
var g_lenOfPitches = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];

// Arrays containing the occurrences of each note duration and of each rest duration
var g_noteLengths = [];
var g_restLengths = [];

// Array with the note names used for output
var g_noteNames = ["C", "C#/Db", "D", "D#/Eb", "E", "F", "F#/Gb", "G", "G#/Ab", "A", "A#/Bb", "B"];

// variables with totals
var g_numOfPages = 0;
var g_numOfBars  = 0;
var g_numOfParts = 0;
var g_lenOfNotes = 0;
var g_lenOfRests = 0;
var g_numOfNotes = 0;
var g_numOfRests = 0;

var g_form;			// the UI dialog box

//---------------------------------------------------------
//	init()
//	this function will be called on startup of mscore
//---------------------------------------------------------

function init()
{
}

//-------------------------------------------------------------------
//	run()
//	this function will be called when activating the plugin menu entry
//
//	global Variables:
//	pluginPath - contains the plugin path; file separator is "/"
//-------------------------------------------------------------------

function run()
{	var		chordnote, staff, voice;
	var		cursor;
	var		idx;
	var		length, note, pitch;
	var		numOfChordNotes;
	var		text;

// this does not seem to work!
//	if(curScore == undefined || curScore == null)
//	{	QMessageBox.critical(null, "Statistics Plugin Error", "No score is open");
//		return;
//	}
	if(curScore.parts != undefined)
		g_numOfParts	= curScore.parts();
	if(curScore.page != undefined)
		g_numOfPages	= curScore.pages();
	if(curScore.measures != undefined)
		g_numOfBars		= curScore.measures();
	cursor			= new Cursor(curScore);

	// for each cursor step of each voice of each staff
	for (staff = 0; staff < curScore.staves; ++staff)
	{	cursor.staff = staff;
		for (voice = 0; voice < 3; voice++)
		{	cursor.voice = voice;
			cursor.rewind();  // set cursor to first chord/rest
			while (!cursor.eos())
			{	// if cursor is at a chord, get chord duration
				if (cursor.isChord())
				{	length			= cursor.chord().tickLen;
					numOfChordNotes	= cursor.chord().notes();
// MuseScore concept of chord is meaningless musically
//					if(numOfChordNotes > 1)
//						g_numOfChords++;
					g_numOfNotes += numOfChordNotes;
					// enumerate chord notes and get the pitch and the duration of each
					for (chordnote = 0; chordnote < numOfChordNotes; chordnote++)
					{	note	= cursor.chord().note(chordnote);
						pitch	= note.pitch % 12;
						g_numOfPitches[pitch]++;
						g_lenOfPitches[pitch] += length;
						g_lenOfNotes += length;
						if(g_noteLengths[length] == undefined)
							g_noteLengths[length] = 1;
						else
							g_noteLengths[length]++;
					}
				}
				// if cursor is at a rest, get its duration
// Version 0.9.5 does not have access to rests!
				else if (cursor.rest != undefined && cursor.isRest())
				{	g_numOfRests++;
					length = cursor.rest().tickLen;
					g_lenOfRest += length;
					if(g_restLengths[length] == undefined)
						g_restLengths[length] = 1;
					else
						g_restLengths[length]++;
				}
				else
				{	g_numOfRests++;
				}
				cursor.next();
			}
		}
	}

	// create UI
	var loader = new QUiLoader(null);
	var file   = new QFile(pluginPath + "/statistics.ui");
	file.open(QIODevice.OpenMode(QIODevice.ReadOnly, QIODevice.Text));
	g_form = loader.load(file, null);
	g_form.pushSave.clicked.connect(dlgSave);
	g_form.pushOk.clicked.connect(dlgAccept);

	// fill summary
	if(g_numOfParts != 0)
		g_form.listParts = g_numOfParts;
	if(g_numOfPages != 0)
		g_form.listPages = g_numOfPages;
	if(g_numOfBars != 0)
		g_form.listBars = g_numOfBars;

	// fill number of occurrences and durations of pitches
	g_form.lName0.setText   ("<b>" + g_noteNames[0] + "</b>");
	g_form.lOccurr0.setText ("" + g_numOfPitches[0]);
	g_form.lLen0.setText    ("" + g_lenOfPitches[0]);
	g_form.lName1.setText   ("<b>" + g_noteNames[1] + "</b>");
	g_form.lOccurr1.setText ("" + g_numOfPitches[1]);
	g_form.lLen1.setText    ("" + g_lenOfPitches[1]);
	g_form.lName2.setText   ("<b>" + g_noteNames[2] + "</b>");
	g_form.lOccurr2.setText ("" + g_numOfPitches[2]);
	g_form.lLen2.setText    ("" + g_lenOfPitches[2]);
	g_form.lName3.setText   ("<b>" + g_noteNames[3] + "</b>");
	g_form.lOccurr3.setText ("" + g_numOfPitches[3]);
	g_form.lLen3.setText    ("" + g_lenOfPitches[3]);
	g_form.lName4.setText   ("<b>" + g_noteNames[4] + "</b>");
	g_form.lOccurr4.setText ("" + g_numOfPitches[4]);
	g_form.lLen4.setText    ("" + g_lenOfPitches[4]);
	g_form.lName5.setText   ("<b>" + g_noteNames[5] + "</b>");
	g_form.lOccurr5.setText ("" + g_numOfPitches[5]);
	g_form.lLen5.setText    ("" + g_lenOfPitches[5]);
	g_form.lName6.setText   ("<b>" + g_noteNames[6] + "</b>");
	g_form.lOccurr6.setText ("" + g_numOfPitches[6]);
	g_form.lLen6.setText    ("" + g_lenOfPitches[6]);
	g_form.lName7.setText   ("<b>" + g_noteNames[7] + "</b>");
	g_form.lOccurr7.setText ("" + g_numOfPitches[7]);
	g_form.lLen7.setText    ("" + g_lenOfPitches[7]);
	g_form.lName8.setText   ("<b>" + g_noteNames[8] + "</b>");
	g_form.lOccurr8.setText ("" + g_numOfPitches[8]);
	g_form.lLen8.setText    ("" + g_lenOfPitches[8]);
	g_form.lName9.setText   ("<b>" + g_noteNames[9] + "</b>");
	g_form.lOccurr9.setText ("" + g_numOfPitches[9]);
	g_form.lLen9.setText    ("" + g_lenOfPitches[9]);
	g_form.lName10.setText  ("<b>" + g_noteNames[10] + "</b>");
	g_form.lOccurr10.setText("" + g_numOfPitches[10]);
	g_form.lLen10.setText   ("" + g_lenOfPitches[10]);
	g_form.lName11.setText  ("<b>" + g_noteNames[11] + "</b>");
	g_form.lOccurr11.setText("" + g_numOfPitches[11]);
	g_form.lLen11.setText   ("" + g_lenOfPitches[11]);

	// build and show summary of note and rest length distribution
	text = "";
	for(idx in g_noteLengths)
	{	text += "" + idx + ":\t" + g_noteLengths[idx] + "\n";
	}
	g_form.listNoteLen.setPlainText(text);
	text = "";
	for(idx in g_restLengths)
	{	text += "" + idx + ":\t" + g_restLengths[idx] + "\n";
	}
	g_form.listRestLen.setPlainText(text);

	// show note and rest totals
	g_form.listNoteTotNum.setText("" + g_numOfNotes);
	g_form.listRestTotNum.setText("" + g_numOfRests);
	g_form.listNoteTotLen.setText("" + g_lenOfNotes);
	if(g_lenToRest != 0)
		g_form.listRestTotLen.setText("" + g_lenOfRests);

	g_form.show();								// show the dlg
}

//---------------------------------------------------------
//	save()
//	called when user presses the "Save" button
//	saves the collected data in a .CSV text file
//---------------------------------------------------------

function dlgSave()
{	// open a file selection dlg
	var fName = QFileDialog.getSaveFileName(g_form, "Select .csv file to create",
			".", "CSV file (*.csv)", 0);
	if(fName == null)
		return;

	// open data file as a text stream
	var file = new QFile(fName);
	if(file.exists())
		file.remove();
	if( !file.open(QIODevice.ReadWrite) )
	{	QMessageBox.critical(g_form, "File Error", "Could not create csv file " + fName);
		return;
	}
	var textStream = new QTextStream(file);

	// output summary
	textStream.writeString("\"Parts:\"," + g_numOfParts + "\n");
	textStream.writeString("\"Pages:\"," + g_numOfPages + "\n");
	textStream.writeString("\"Bars:\","  + g_numOfBars  + "\n");

	// output pitch occurrences and durations
	textStream.writeString("\"" + g_noteNames[0] + "\"," + g_numOfPitches[0] + "," + g_lenOfPitches[0] + "\n");
	textStream.writeString("\"" + g_noteNames[1] + "\"," + g_numOfPitches[1] + "," + g_lenOfPitches[1] + "\n");
	textStream.writeString("\"" + g_noteNames[2] + "\"," + g_numOfPitches[2] + "," + g_lenOfPitches[2] + "\n");
	textStream.writeString("\"" + g_noteNames[3] + "\"," + g_numOfPitches[3] + "," + g_lenOfPitches[3] + "\n");
	textStream.writeString("\"" + g_noteNames[4] + "\"," + g_numOfPitches[4] + "," + g_lenOfPitches[4] + "\n");
	textStream.writeString("\"" + g_noteNames[5] + "\"," + g_numOfPitches[5] + "," + g_lenOfPitches[5] + "\n");
	textStream.writeString("\"" + g_noteNames[6] + "\"," + g_numOfPitches[6] + "," + g_lenOfPitches[6] + "\n");
	textStream.writeString("\"" + g_noteNames[7] + "\"," + g_numOfPitches[7] + "," + g_lenOfPitches[7] + "\n");
	textStream.writeString("\"" + g_noteNames[8] + "\"," + g_numOfPitches[8] + "," + g_lenOfPitches[8] + "\n");
	textStream.writeString("\"" + g_noteNames[9] + "\"," + g_numOfPitches[9] + "," + g_lenOfPitches[9] + "\n");
	textStream.writeString("\"" + g_noteNames[10]+ "\"," + g_numOfPitches[10]+ "," + g_lenOfPitches[10]+ "\n");
	textStream.writeString("\"" + g_noteNames[11]+ "\"," + g_numOfPitches[11]+ "," + g_lenOfPitches[11]+ "\n");

	// output totals
	textStream.writeString("\"Total number of notes:\","   + g_numOfNotes + "\n");
	textStream.writeString("\"Total duration of notes:\"," + g_lenOfNotes + "\n");
	textStream.writeString("\"Total number of rests:\","   + g_numOfRests + "\n");
	textStream.writeString("\"Total duration of rests:\"," + g_lenOfRests + "\n");

	// output summary of note and rest length distribution
	textStream.writeString("\"Length occurences of notes\"\n");
	for(idx in g_noteLengths)
		textStream.writeString("" + idx + "," + g_noteLengths[idx] + "\n");
	textStream.writeString("\"Length occurences of rests\"\n");
	for(idx in g_restLengths)
		textStream.writeString("" + idx + "," + g_restLengths[idx] + "\n");

	file.close();
}

//---------------------------------------------------------
//	dlgAccept()
//	called when user presses the "OK" button
//---------------------------------------------------------

function dlgAccept()
{	g_form.accept();
}

//---------------------------------------------------------
//    menu:  defines were the function will be placed
//           in the MuseScore menu structure
//---------------------------------------------------------

var mscorePlugin = {
      menu: 'Plugins.Statistics',
      init: init,
      run:  run
      };

mscorePlugin;
